<?php

/*
$Id: cachemanager.class.php 746 2007-01-31 04:02:50Z lkppo $

Copyright (c) 2002, Hendrik Mans <hendrik@mans.de>
Copyright 2003 Carlo Zottmann
Copyright 2002-2005, David DELON
Copyright 2002-2004, 2006 Charles NPOTE
Copyright 2002-2003, Patrick PAUL
Copyright 2003, Eric DELORD
Copyright 2003, Eric FELDSTEIN
Copyright 2004-2006, Jean-Christophe ANDR
Copyright 2005-2006, Didier LOISEAU
Copyright 2006-2007, Stphane AULERY

All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

class CacheManager
{
	// VARIABLES
	
	var $IndexExistingPages = array();
	var $IndexByTag = array();
	var $IndexById = array();
	var $Pages = array();



	// CONSTRUCTOR
	
	function CacheManager()
	{
		$this->UnCacheAllPages();
	}


	// INDEX DE TOUTES LES PAGES

	function CacheIndexAllPage($Index)
	{
		// Build an array indexed by page tag for maximum performance
		foreach ($Index as $page) $this->IndexExistingPages[strtolower($page['tag'])] = $page['tag'];
	}

	function CacheExistingPage($Tag)
	{
		if (!isset($this->IndexExistingPages[strtolower($Tag)])) $this->IndexExistingPages[strtolower($Tag)] = $Tag;
	}

	function RemoveExistingPage($Tag)
	{
		unset($this->IndexExistingPages[strtolower($Tag)]);

		// Vrifie si la page existe dj dans le cache
		if (!isset($this->IndexByTag[$Tag])) $this->UnCachePage($Tag);
	}

	/**
	 * Checks the existence of a page
	 * @param string $tag The tag (name) of the page to check
	 * @param boolean $casse_respect, FALSE by default
	 * @return boolean true iff a page named $tag exists
	 */
	function PageExists($tag, $casse_respect = FALSE)
	{		
		// If the given tag is a valid index, then the page exists
		if (isset($this->IndexExistingPages[strtolower($tag)])) {
			if ($casse_respect == TRUE) {
				if ($this->IndexExistingPages[strtolower($tag)] == $tag) return TRUE;
				else return FALSE;
			}
			else return TRUE;
		}
		else return FALSE;
	}

	// Renvoi un tag existant avec la bonne casse
	function GetExistingPageTag($Tag)
	{
		if (isset($this->IndexExistingPages[strtolower($Tag)])) return $this->IndexExistingPages[strtolower($Tag)];
		else return $Tag;
	}

	// PAGES

	function CachePage($page)
	{
		// Vrifie si la page existe dj dans le cache
		if (!isset($this->IndexByTag[$page["tag"]])) {
			// Vrifie si la page pass en paramtre est complte
			if(count($page) == 8 or !isset($page["id"])) {

				$this->Pages[$page["id"]] = $page;		
				
				$this->IndexByTag[$page["tag"]] = $page["id"];
				$this->IndexById[$page["id"]] = $page["id"];
			}
		}
	}


	/*
	 * Supprime une entre du cache
	 */
	function UnCachePage($RemovePage)
	{
		// Si la valeur fourni est un numro
		if (is_numeric($RemovePage)) {
			// Vrifie si la page existe dj dans le cache
			if (isset($this->IndexById[$RemovePage])) {
				
				$Tag = $this->Pages[$this->IndexById[$RemovePage]]["tag"];
				
				unset($this->Pages[$RemovePage]);
				unset($this->IndexByTag[$Tag]);
				unset($this->IndexById[$RemovePage]);

				unset($Tag);
			}
		}
		// Si c'est un nom de page non nul
		elseif (strlen($RemovePage) > 0) {
			// Vrifie si la page existe dj dans le cache
			if (isset($this->IndexByTag[$RemovePage])) {

				$Id = $this->Pages[$this->IndexByTag[$RemovePage]]["id"];

				unset($this->Pages[$Id]);
				unset($this->IndexByTag[$RemovePage]);
				unset($this->IndexById[$Id]);

				unset($Id);
			}
		}
	}

	/*
	 * Vide le cache
	 */
	function UnCacheAllPages()
	{
		$this->IndexExistingPages = array();
		$this->IndexByTag = array();
		$this->IndexById = array();
		$this->Pages = array();
	}

	/*
	 * Extraire une page du cache
	 */
	function GetCachedPage($PageRequired)
	{
		// Si la valeur fourni est un numro
		if (is_numeric($PageRequired)) {
			if (isset($this->IndexById[$PageRequired]))
				return $this->Pages[$this->IndexById[$PageRequired]];
		}
		// Si c'est un nom de page non nul
		elseif (strlen($PageRequired) > 0) {
			if (isset($this->IndexByTag[$PageRequired])) return $this->Pages[$this->IndexByTag[$PageRequired]];
		}
	}
}

?>
