<?php

/*
$Id: wiki.class.php 746 2007-01-31 04:02:50Z lkppo $

Copyright (c) 2002, Hendrik Mans <hendrik@mans.de>
Copyright 2003 Carlo Zottmann
Copyright 2002-2005, David DELON
Copyright 2002-2004, 2006 Charles NPOTE
Copyright 2002-2003, Patrick PAUL
Copyright 2003, Eric DELORD
Copyright 2003, Eric FELDSTEIN
Copyright 2004-2006, Jean-Christophe ANDR
Copyright 2005-2006, Didier LOISEAU
Copyright 2006-2007, Stphane AULERY
Copyright 2007,  Xavier GARREAU

All rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

class Wiki
{
	// VARIABLES
	var $dbObject;
	var $page;
	var $parameter = array();
	var $queryLog = array();
	var $interWiki = array();
	var $VERSION;
	var $CookiePath = '/';
	var $inclusions = array();

	/**
	 * an array containing all the actions that are implemented by an object
	 * @access private
	 */
	var $actionObjects;
	var $themeManager;
	var $Cache;



	// CONSTRUCTOR
	
	function Wiki($config)
	{
		$this->config = $config;

		// if in debug mode, activate full error reporting
		if ($this->GetConfigValue('debug') === 'yes') error_reporting(E_ALL);

		$this->dbObject = database_constructor ($this->GetConfigValue('dbtype'), $this->config);

		// check for database access
		if ($this->dbObject) {
			if (!$this->dbObject->query("SELECT 0")) {
				$this->dbObject->close();
				$this->dbObject = NULL;
			}
		}
		else {
			echo "<p>Pour des raisons ind&eacute;pendantes de notre volont&eacute;, ".
			"le contenu de ce Wiki est temporairement inaccessible. Veuillez ".
			"r&eacute;essayer ult&eacute;rieurement, merci de votre ".
			"compr&eacute;hension.</p>";

			// Log error (useful to find the buggy server in a load balancing platform)
			if ($this->GetConfigValue('debug') === 'yes') trigger_error("WikiNi : DB connection failed");
			exit;
		}

		$this->VERSION = WAKKA_VERSION;

		$this->themeManager = &new ThemeManager($this->GetConfigValue('themes_path'),
							$this->GetConfigValue('themes_dir_url'));
	}



	// DATABASE

	// Run a Query
	function Query($query)
	{
		if($this->GetConfigValue("debug")) $start = $this->GetMicroTime();

		if (!$result = $this->dbObject->query($query))
		{
			ob_end_clean();
			die("Query failed : ".$query."\n(".$this->dbObject->error_string().")");
		}

		if($this->GetConfigValue("debug"))
		{
			$time = $this->GetMicroTime() - $start;
			$this->queryLog[] = array(
				"query"		=> $query,
				"time"		=> $time);
		}

		return $result;
	}

	// Run a Query without return value and fetching and buffering the result rows automatically
	function QueryOnly($query)
	{
		if($this->GetConfigValue("debug")) $start = $this->GetMicroTime();

		// Execute
		$resulat = $this->dbObject->unbuffered_query($query);

		if ($resulat == FALSE)
		{
			ob_end_clean();
			die("Query failed : ".$query."\n(".$this->dbObject->error_string().")");
		}

		if($this->GetConfigValue("debug"))
		{
			$time = $this->GetMicroTime() - $start;
			$this->queryLog[] = array(
				"query"		=> $query,
				"time"		=> $time);
		}

		return $resulat;
	}

	// Load a single row
	function LoadSingle($query)
	{
		if ($Result = $this->Query($query)) { 
			$data[] = $this->dbObject->fetch_assoc($Result);
			
			$this->dbObject->free_result($Result);

			return $data[0];
		}
	}

	// Load all rows
	function LoadAll($query)
	{
		$data=array();
		if ($r = $this->Query($query))
		{
			while ($row = $this->dbObject->fetch_assoc($r)) $data[] = $row;
			$this->dbObject->free_result($r);
		}

		return $data;
	}



	// MISC
	
	function GetMicroTime()
	{
		list($usec, $sec) = explode(" ",microtime()); return ((float)$usec + (float)$sec);
	}

	function IncludeBuffered($filename, $notfoundText = "", $vars = "", $path = "")
	{
		if ($path) $dirs = explode(":", $path);
		else $dirs = array("");

		foreach($dirs as $dir)
		{
			if ($dir) $dir .= "/";
			$fullfilename = $dir.$filename;
			if (file_exists($fullfilename))
			{
				if (is_array($vars)) extract($vars);

				ob_start();
				include($fullfilename);
				$output = ob_get_contents();
				ob_end_clean();
				return $output;
			}
		}
		if ($notfoundText) return $notfoundText;
		else return false;
	}

	function &GetThemeManager()
	{
		return $this->themeManager;
	}



	// GET SOME VARIABLES
	
	function GetPageTag()
	{
		return $this->page["tag"];
	}

	function GetPageTime()
	{
		return $this->page["time"];
	}

	function GetMethod()
	{
		return $this->method;
	}
	function GetConfigValue($name) {
		return isset($this->config[$name]) ? trim($this->config[$name]) : '';
	}

	function GetWakkaName()
	{
		return $this->GetConfigValue("wakka_name");
	}

	function GetWakkaVersion()
	{
		return $this->VERSION;
	}

	function GetWikiNiVersion()
	{
		return WIKINI_VERSION;
	}



	// TRIPLES
	
	/**
	 * Retrieves all the values for a given couple (resource, property)
	 * @param string $resource The resource of the triples
	 * @param string $property The property of the triple to retrieve
	 * @param string $re_prefix The prefix to add to $resource (defaults to THISWIKI_PREFIX)
	 * @param string $prop_prefix The prefix to add to $property (defaults to WIKINI_VOC_PREFIX)
	 * @return array An array of the retrieved values, in the form
	 * array(
	 * 	0 => array(id = 7 , 'value' => $value1),
	 *  1 => array(id = 34, 'value' => $value2),
	 *  ...
	 * )
	 */
	function GetAllTriplesValues($resource, $property, $re_prefix = THISWIKI_PREFIX, $prop_prefix = WIKINI_VOC_PREFIX)
	{
		$sql = 'SELECT id, value FROM ' . $this->GetConfigValue('table_prefix') . 'triples '
			. 'WHERE resource = "' . $this->dbObject->escape_string($re_prefix . $resource) . '" '
			. 'AND property = "' . $this->dbObject->escape_string($prop_prefix . $property) . '" ';

		return $this->LoadAll($sql);
	} 
		
	/**
	 * Retrieves a single value for a given couple (resource, property)
	 * @param string $resource The resource of the triples
	 * @param string $property The property of the triple to retrieve
	 * @param string $re_prefix The prefix to add to $resource (defaults to <tt>THISWIKI_PREFIX</tt>)
	 * @param string $prop_prefix The prefix to add to $property (defaults to <tt>WIKINI_VOC_PREFIX</tt>)
	 * @return string The value corresponding to ($resource, $property) or null if
	 * there is no such couple in the triples table.
	 */
	function GetTripleValue($resource, $property, $re_prefix = THISWIKI_PREFIX, $prop_prefix = WIKINI_VOC_PREFIX)
	{
		$res = $this->GetAllTriplesValues($resource, $property, $re_prefix, $prop_prefix);
		if ($res) return $res[0]['value'];

		return null;
	}
	
	/**
	 * Checks whether a triple exists or not
	 * @param string $resource The resource of the triple to find
	 * @param string $property The property of the triple to find
	 * @param string $value The value of the triple to find
	 * @param string $re_prefix The prefix to add to $resource (defaults to <tt>THISWIKI_PREFIX</tt>)
	 * @param string $prop_prefix The prefix to add to $property (defaults to <tt>WIKINI_VOC_PREFIX</tt>)
	 * @param int The id of the found triple or 0 if there is no such triple. 
	 */
	function TripleExists($resource, $property, $value, $re_prefix = THISWIKI_PREFIX, $prop_prefix = WIKINI_VOC_PREFIX)
	{
		$sql = 'SELECT id FROM ' . $this->GetConfigValue('table_prefix') . 'triples '
			. 'WHERE resource = "' . $this->dbObject->escape_string($re_prefix . $resource) . '" '
			. 'AND property = "' . $this->dbObject->escape_string($prop_prefix . $property) . '" '
			. 'AND value = "' . $this->dbObject->escape_string($value) . '"';
		$res = $this->LoadSingle($sql);
		if (!$res) return 0;

		return $res['id'];
	}
	
	/**
	 * Inserts a new triple ($resource, $property, $value) in the triples' table
	 * @param string $resource The resource of the triple to insert
	 * @param string $property The property of the triple to insert
	 * @param string $value The value of the triple to insert
	 * @param string $re_prefix The prefix to add to $resource (defaults to <tt>THISWIKI_PREFIX</tt>)
	 * @param string $prop_prefix The prefix to add to $property (defaults to <tt>WIKINI_VOC_PREFIX</tt>)
	 * @return int An error code: 0 (success), 1 (failure) or 3 (already exists)
	 */
	function InsertTriple($resource, $property, $value, $re_prefix = THISWIKI_PREFIX, $prop_prefix = WIKINI_VOC_PREFIX)
	{
		if ($this->TripleExists($resource, $property, $value, $re_prefix, $prop_prefix))
		{
			return 3;
		}
		$sql = 'INSERT INTO ' . $this->GetConfigValue('table_prefix') . 'triples (resource, property, value) '
			. 'VALUES ("' . $this->dbObject->escape_string($re_prefix . $resource) . '", "'
				. $this->dbObject->escape_string($prop_prefix . $property) . '", "'
				. $this->dbObject->escape_string($value) . '")';

		return $this->QueryOnly($sql) ? 0 : 1;
	}
	
	/**
	 * Updates a triple ($resource, $property, $value) in the triples' table
	 * @param string $resource The resource of the triple to UPDATE
	 * @param string $property The property of the triple to UPDATE
	 * @param string $oldvalue The old value of the triple to UPDATE
	 * @param string $newvalue The new value of the triple to UPDATE
	 * @param string $re_prefix The prefix to add to $resource (defaults to <tt>THISWIKI_PREFIX</tt>)
	 * @param string $prop_prefix The prefix to add to $property (defaults to <tt>WIKINI_VOC_PREFIX</tt>)
	 * @return int An error code: 0 (succs), 1 (chec),
	 * 		2 ($resource, $property, $oldvalue does not exist)
	 * 		or 3 ($resource, $property, $newvalue already exists)
	 */
	function UpdateTriple($resource, $property, $oldvalue, $newvalue, $re_prefix = THISWIKI_PREFIX, $prop_prefix = WIKINI_VOC_PREFIX)
	{
		$id = $this->TripleExists($resource, $property, $oldvalue, $re_prefix, $prop_prefix);
		if (!$id) return 2;
		if ($this->TripleExists($resource, $property, $newvalue, $re_prefix, $prop_prefix))
		{
			return 3;
		}
		$sql = 'UPDATE ' . $this->GetConfigValue('table_prefix') . 'triples '
			. 'SET value = "' . $this->dbObject->escape_string($newvalue) . '" '
			. 'WHERE id = ' . $id;

		return $this->QueryOnly($sql) ? 0 : 1;
	}
	
	/**
	 * Deletes a triple ($resource, $property, $value) from the triples' table
	 * @param string $resource The resource of the triple to delete
	 * @param string $property The property of the triple to delete
	 * @param string $value The value of the triple to delete. If set to <tt>null</tt>,
	 * deletes all the triples corresponding to ($resource, $property). (defaults to <tt>null</tt>)
	 * @param string $re_prefix The prefix to add to $resource (defaults to <tt>THISWIKI_PREFIX</tt>)
	 * @param string $prop_prefix The prefix to add to $property (defaults to <tt>WIKINI_VOC_PREFIX</tt>)
	 */
	function DeleteTriple($resource, $property, $value = null, $re_prefix = THISWIKI_PREFIX, $prop_prefix = WIKINI_VOC_PREFIX)
	{
		$sql = 'DELETE FROM ' . $this->GetConfigValue('table_prefix') . 'triples '
			. 'WHERE resource = "' . $this->dbObject->escape_string($re_prefix . $resource) . '" '
			. 'AND property = "' . $this->dbObject->escape_string($prop_prefix . $property) . '" ';
		if ($value !== null) $sql .= 'AND value = "' . $this->dbObject->escape_string($value) . '"';

		$this->QueryOnly($sql);
	}



	// INCLUSIONS
	
	/**
	 * Enrgistre une nouvelle inclusion dans la pile d'inclusions.
	 * 
	 * @param string $pageTag Le nom de la page qui va tre inclue
	 * @return int Le nombre d'lments dans la pile
	 */
	function RegisterInclusion($pageTag)
	{
		return array_unshift($this->inclusions, strtolower(trim($pageTag)));
	} 
	/**
	 * Retire le dernier lment de la pile d'inclusions.
	 * 
	 * @return string Le nom de la page dont l'inclusion devrait se terminer.
	 * null s'il n'y a plus d'inclusion dans la pile.
	 */
	function UnregisterLastInclusion()
	{
		return array_shift($this->inclusions);
	}

	/**
	 * Renvoie le nom de la page en cours d'inclusion.
	 * 
	 * @example // dans le cas d'une action comme l'ActionEcrivezMoi
	 * if($inc = $this->CurrentInclusion() && strtolower($this->GetPageTag()) != $inc)
	 * 	echo 'Cette action ne peut tre appele depuis une page inclue';
	 * @return string Le nom (tag) de la page (en minuscules)
	 * false si la pile est vide.
	 */
	function GetCurrentInclusion()
	{
		return isset($this->inclusions[0]) ? $this->inclusions[0]: false ;
	}

	/**
	 * Vrifie si on est  l'intrieur d'une inclusion par $pageTag (sans tenir compte de la casse)
	 * 
	 * @param string $pageTag Le nom de la page  vrifier
	 * @return bool True si on est  l'intrieur d'une inclusion par $pageTag (false sinon)
	 */
	function IsIncludedBy($pageTag)
	{
		return in_array(strtolower($pageTag), $this->inclusions);
	}

	/**
	 * 
	 * @return array La pile d'inclusions
	 * L'lment 0 sera la dernire inclusion, l'lment 1 sera son parent et ainsi de suite.
	 */
	function GetAllInclusions()
	{
		return $this->inclusions;
	}

	/**
	 * Remplace la pile des inclusions par une nouvelle pile (par dfaut une pile vide)
	 * Permet de formatter une page sans tenir compte des inclusions prcdentes.
	 * 
	 * @param array $ La nouvelle pile d'inclusions.
	 * L'lment 0 doit reprsenter la dernire inclusion, l'lment 1 son parent et ainsi de suite.
	 * @return array L'ancienne pile d'inclusions, avec les noms des pages en minuscules.
	 */
	function SetInclusions($pile = array())
	{
		$temp = $this->inclusions;
		$this->inclusions = $pile;

		return $temp;
	} 



	// PAGES

	/*
	 * Stocke par rfrence l'objet qui fournie le cache
	 */
	function SetCache(&$cache)
	{
		$this->Cache = &$cache;
	}

	/**
	 * Checks the existence of a page
	 * @param string $tag The tag (name) of the page to check
	 * @param boolean $casse_respect, FALSE by default
	 * @return boolean true iff a page named $tag exists
	 */
	function PageExists($tag, $casse_respect = FALSE)
	{
		return $this->Cache->PageExists($tag, $casse_respect);
	}

	// Renvoi un tag existant avec la bonne casse
	function GetExistingPageTag($tag)
	{
		return $this->Cache->GetExistingPageTag($tag);
	}
	
	function LoadPage($tag, $time = "", $cache = 1)
	{
		// retrieve from cache
		if (!$time && $cache && ($cachedPage = $this->Cache->GetCachedPage($tag))) { $page = $cachedPage;}

		// load page
		if (!isset($page)) {
			$page = $this->LoadSingle("SELECT * "
					."FROM ".$this->GetConfigValue("table_prefix")."pages "
					."WHERE tag = '".$this->dbObject->escape_string($tag)."' "
					.($time ? "AND time = '".$this->dbObject->escape_string($time)."'" : "AND latest = 'Y'")
					." LIMIT 1");

			// cache result
			$this->Cache->CachePage($page);
		}

		return $page;
	}

	function SetPage($page)
	{
		$this->page = $page;
	}
	
	function LoadPageById($id, $cache = 1)
	{
		// retrieve from cache
		if ($cache && ($cachedPage = $this->Cache->GetCachedPage($id))) $page = $cachedPage;
		// load page
		if (!isset($page)) {
			$page = $this->LoadSingle("SELECT * "
					."FROM ".$this->GetConfigValue("table_prefix")."pages "
					."WHERE id = '".$this->dbObject->escape_string($id)
					."' LIMIT 1");
			
			$this->Cache->CachePage($page);
		}

		return $page;
	}
	
	function LoadRevisions($page)
	{
		return $this->LoadAll("SELECT * "
			."FROM ".$this->GetConfigValue("table_prefix")."pages "
			."WHERE tag = '".$this->dbObject->escape_string($page)."' "
			."ORDER BY time DESC");
	}
	
	function LoadPagesLinkingTo($tag)
	{
		return $this->LoadAll("SELECT DISTINCT from_tag AS tag "
				."FROM ".$this->GetConfigValue("table_prefix")."links "
				."WHERE to_tag = '".$this->dbObject->escape_string($tag)."' "
				."ORDER BY from_tag");
	}
	
	function LoadRecentlyChanged($limit = 50)
	{
		$limit= (int) $limit;
		if ($pages = $this->LoadAll("SELECT tag, time, user, owner "
					."FROM ".$this->GetConfigValue("table_prefix")."pages "
					."WHERE latest = 'Y' "
					."AND comment_on = '' "
					."ORDER BY time DESC "
					."LIMIT $limit"))
		{
			return $pages;
		}
	}
	
	function LoadAllPages()
	{
		return $this->LoadAll("SELECT * "
				."FROM ".$this->GetConfigValue("table_prefix")."pages "
				."WHERE latest = 'Y' "
				."ORDER BY tag");
	}
	
	function FullTextSearch($phrase)
	{
        switch ($this->GetConfigValue('dbtype')) {
            case 'mysql' :
                $sql = "SELECT * "
    				."FROM ".$this->GetConfigValue("table_prefix")."pages "
    				."WHERE latest = 'Y' "
    				."AND MATCH(tag, body) "
    				."AGAINST('".$this->dbObject->escape_string($phrase)."')";
    			break;
            case 'sqlite' :
                $sql = "SELECT * "
    				."FROM ".$this->GetConfigValue("table_prefix")."pages "
    				."WHERE latest = 'Y' "
    				."AND tag||body "
    				."LIKE '%".$this->dbObject->escape_string($phrase)."%'";
    			break;
    	}
        return $this->LoadAll($sql);
	}

	function LoadWantedPages()
	{ 
		return $this->LoadAll("SELECT DISTINCT T_Link.to_tag AS tag, count(T_Link.from_tag) AS count "
			."FROM ".$this->GetConfigValue("table_prefix")."links AS T_Link "
			."LEFT JOIN ".$this->GetConfigValue("table_prefix")."pages "
			."ON T_Link.to_tag = ".$this->GetConfigValue("table_prefix")."pages.tag "
			."WHERE ".$this->GetConfigValue("table_prefix")."pages.tag IS NULL "
			."GROUP BY to_tag ORDER BY count DESC, to_tag ASC");
	}


	function LoadOrphanedPages()
	{
		return $this->LoadAll("SELECT DISTINCT tag "
			."FROM ".$this->GetConfigValue("table_prefix")."pages AS T_Page "
			."LEFT JOIN ".$this->GetConfigValue("table_prefix")."links AS T_Link "
			."ON T_Page.tag = T_Link.to_tag "
			."WHERE T_Link.to_tag IS NULL "
			."AND T_Page.comment_on = '' "
			."AND T_Page.latest = 'Y' ORDER BY tag");
	}
	
	function IsOrphanedPage($tag)
	{
		return $this->LoadAll("SELECT DISTINCT tag "
			."FROM ".$this->GetConfigValue('table_prefix')."pages AS T_Page "
			."LEFT JOIN ".$this->GetConfigValue('table_prefix')."links as T_Link "
			."ON T_Page.tag = T_Link.to_tag "
			."WHERE T_Link.to_tag IS NULL "
			."AND T_Page.latest = 'Y' "
			."AND tag = '".$this->dbObject->escape_string($tag)."'");
	}
	
	function DeletePage($tag) {
		$this->QueryOnly("DELETE FROM ".$this->GetConfigValue("table_prefix")."pages "
			."WHERE tag='".$this->dbObject->escape_string($tag)."' "
			."OR comment_on='".$this->dbObject->escape_string($tag)."'");
		$this->QueryOnly("DELETE FROM ".$this->GetConfigValue("table_prefix")."links "
			."WHERE from_tag='".$this->dbObject->escape_string($tag)."' ");
		$this->QueryOnly("DELETE FROM ".$this->GetConfigValue("table_prefix")."acls "
			."WHERE page_tag='".$this->dbObject->escape_string($tag)."' ");
		$this->QueryOnly("DELETE FROM ".$this->GetConfigValue("table_prefix")."referrers "
			."WHERE page_tag='".$this->dbObject->escape_string($tag)."' ");

		$this->Cache->RemoveExistingPage($tag);
	}
	
	function SavePage($tag, $body, $comment_on = "")
	{
		// get current user
		$user = $this->GetUserName();

		// check write privilege
		if ($comment_on ? $this->HasAccess("comment", $comment_on) : $this->HasAccess("write", $tag)) {
			// Is a new page ?
			if ($this->PageExists($tag) == FALSE) {
				// current user is owner ; if user is logged in ! otherwise, no owner.
				if ($this->GetUser()) $owner = $user;
				else $owner = '';

				$this->Cache->CacheExistingPage($tag);
			}
			else {
				$oldPage = $this->LoadPage($tag);

				// aha ! page isn't new. keep owner!
				$owner = $oldPage["owner"];

				// ...and comment_on, eventualy?
				if ($comment_on == '') $comment_on = $oldPage['comment_on'];

				// set all other revisions to old
				$this->QueryOnly("UPDATE ".$this->GetConfigValue("table_prefix")."pages "
					."SET latest = 'N' "
					."WHERE tag = '".$this->dbObject->escape_string($tag)."'");

				// delete olds links about this page
				$this->QueryOnly("DELETE FROM ".$this->GetConfigValue("table_prefix")."links "
					."WHERE from_tag = '".$this->dbObject->escape_string($this->GetPageTag())."'");
			}

			// add new revision
			$this->QueryOnly("INSERT INTO ".$this->GetConfigValue("table_prefix")."pages"
			    ."(tag,comment_on,time,owner,user,latest,body) "
				."VALUES ('".$this->dbObject->escape_string($tag)."', ".
				"'".$this->dbObject->escape_string($comment_on)."', ".
				"'".date('Y-m-d H:i:s')."', ".
				"'".$this->dbObject->escape_string($owner)."', ".
				"'".$this->dbObject->escape_string($user)."', ".
				"'Y', ".
				"'".$this->dbObject->escape_string(chop($body))."')");

			// Supprime du Cache l'ancienne version de la page
			$this->Cache->UnCachePage($tag);

			// Update Link Table
			$this->WriteLinkTable($tag);
		}
	}

	/**
	 * Make the purge of page versions that are older than the last version older than 3 "pages_purge_time"
	 * This method permits to allways keep a version that is older than that period.
	 */
	function PurgePages()
	{
		if ($days = $this->GetConfigValue("pages_purge_time")) { // is purge active ?

			// let's search which pages versions we have to remove
			// this is necessary beacause even MySQL does not handel multi-tables deletes before version 4.0
			$wnPages = $this->GetConfigValue('table_prefix') . 'pages';
			$datesub = date('Y-m-d H:i:s', 
                            mktime(date('H'), date('i'), date('s'),
                                   date('m'), date('d')-$days, date('Y')));
			$sql = 'SELECT DISTINCT a.id FROM ' . $wnPages . ' a,' . $wnPages . ' b '
				.'WHERE a.latest = \'N\' '
				.'AND a.time < \''.$datesub.'\' '
				.'AND a.tag = b.tag '
				.'AND a.time < b.time '
				.'AND b.time < \''.$datesub.'\'';
				
			$ids = $this->LoadAll($sql);

			if (count($ids)) { // there are some versions to remove from DB
				// let's build one big request, that's better...
				$sql = 'DELETE FROM '.$this->GetConfigValue('table_prefix').'pages '
					.'WHERE id IN (';
				foreach($ids as $key => $line){
					$sql .= ($key ? ', ':'') . $line['id']; // NB.: id is an int, no need of quotes
				}
				$sql .= ')';
			
				// ... and send it !
				$this->QueryOnly($sql);
			}
		}
	}



	// COOKIES
	
	function SetSessionCookie($name, $value)
	{
		SetCookie($name, $value, 0, $this->CookiePath);
		$_COOKIE[$name] = $value;
	}

	function SetPersistentCookie($name, $value, $remember = 0)
	{
		SetCookie($name, $value, time() + ($remember ? 90*24*60*60 : 60 * 60), $this->CookiePath);
		$_COOKIE[$name] = $value;
	}

	function DeleteCookie($name)
	{
		SetCookie($name, "", 1, $this->CookiePath);
		$_COOKIE[$name] = "";
	}

	function GetCookie($name)
	{
		return $_COOKIE[$name];
	}



	// HTTP / REQUEST / LINK RELATED
	
	function SetMessage($message)
	{
		$_SESSION["message"] = $message;
	}

	function GetMessage()
	{
		if (isset($_SESSION["message"])) $message = $_SESSION["message"];
		else $message = "";
		$_SESSION["message"] = "";

		return $message;
	}

	function Redirect($url)
	{
		header("Location: $url");

		exit;
	}

	// returns just PageName[/method].
	function MiniHref($method = "", $tag = "")
	{
		if (!$tag = trim($tag)) $tag = $this->GetPageTag();
		else $tag = $this->GetExistingPageTag($tag);

		return $tag.($method ? "/".$method : "");
	}

	// returns the full url to a page/method.
	function Href($method = "", $tag = "", $params = "")
	{
		$href = $this->GetConfigValue("base_url").$this->MiniHref($method, $tag);
		if ($params) $href .= ($this->GetConfigValue("rewrite_mode") ? "?" : "&amp;").$params;

		return $href;
	}

	function Link($tag, $method = "", $text = "")
	{
		$tag  = htmlspecialchars($tag); //avoid xss
		$text = htmlspecialchars($text); //paranoiac again
		$text = preg_replace('/&amp;(\\#[xX][a-fA-F0-9]+|\\#[0-9]+|[a-zA-Z0-9]+);/', '&$1;', $text);
		if (!$text) $text = $tag;

		// is this an interwiki link?
		if (preg_match('/^' . WN_INTERWIKI_CAPTURE . '$/s', $tag, $matches))
		{
			if ($tagInterWiki = $this->GetInterWikiUrl($matches[1], $matches[2])) {
				return "<a href=\"$tagInterWiki\">$text (interwiki)</a>";
			}
			else return "<a href=\"$tag\">$text (interwiki inconnu)</a>";
		}
		// is this a full link ? ie, does it contain non alpha-numeric characters ?
		// Note : [:alnum:] is equivalent [0-9A-Za-z]
		//        [^[:alnum:]] means : some caracters other than [0-9A-Za-z]
		// For example : "www.adress.com", "mailto:adress@domain.com", "http://www.adress.com"
		else if (preg_match("/[^[:alnum:]]/", $tag))
		{
			// check for email addresses
			if (preg_match("/^[\w.-]+\@[\w.-]+$/", $tag))
			{
				$tag = "mailto:".$tag;
			}
			// check for protocol-less URLs
			else if (!preg_match("/:\/\//", $tag))
			{
				$tag = "http://".$tag;	//Very important for xss (avoid javascript:() hacking)
			}
			// is this an inline image (text!=tag and url ends png,gif,jpeg)
			if ($text!=$tag and preg_match("/\.(gif|jpeg|png|jpg)$/i",$tag))
			{
				return "<img src=\"$tag\" alt=\"$text\"/>";
			}
			else
			{
				return "<a href=\"$tag\">$text</a>";
			}
		}
		else // it's a Wiki link!
		{

			if ($this->PageExists($tag)) {
				$tag = $this->GetExistingPageTag($tag);

				return "<a href=\"".$this->href($method, $tag)."\">$text</a>";
			}
			else {
				return "<a href=\"".$this->href("edit", $tag)."\" class=\"missingpage\" title=\"Cr&eacute;er cette page\">$text</a>";
			}
		}
	}

	function ComposeLinkToPage($tag, $method = "", $text = "")
	{
		if (!$text) $text = $tag;
		$text = htmlentities($text);

		return '<a href="'.$this->href($method, $tag).'">'.$text.'</a>';
	}

	function IsWikiName($text)
	{
		return preg_match('/^' . WN_WIKI_LINK . '$/', $text);
	}

	/**
	 * Extrait d'un texte les liens au format Wiki qui si trouvent
	 */
	function WikiLinkTraking($texte)
	{
		// Recherche tous les liens au format wiki
		// (ceux entre "" "", les InterWikis, et les normaux)
		if (preg_match_all("/\"\".*?\"\"|"
			."\{\{.*?\}\}|"
			.'\b' . WN_INTERWIKI_LINK . '\b|'
			.'\b' . WN_WIKI_LINK . '\b|'
			."\n/ms", $texte, $ListeDesLiens)) {

			// Parcours toutes les valeurs trouves
			foreach ($ListeDesLiens[0] as $UnLien) {
				// On isole les liens strictement Wiki
				// (ni ceux entre "" "", ni les InterWikis)
				if ($this->IsWikiName($UnLien)) {
					// S'il est connu on le format  la bonne casse
					if ($this->PageExists($UnLien)) $LiensExtraits[] =$this->GetExistingPageTag($UnLien);
					else $LiensExtraits[] = $UnLien;
				}
			}

			if (isset($LiensExtraits)) {
				// Supprime les liens redondants
				$LiensExtraits = array_unique($LiensExtraits);

				// Retour de la liste des liens trouvs
				return $LiensExtraits;
			}
		}
	}

	/**
	 * Enregistre la table des liens d'une page
	 */
	function WriteLinkTable($tag = '')
	{
		// delete olds links about this page
		$this->QueryOnly("DELETE FROM ".$this->GetConfigValue("table_prefix")."links "
			."WHERE from_tag = '".$this->dbObject->escape_string($tag)."'");
		
		$Page =  $this->LoadPage($tag);

		if ($ListeDesLiens = $this->WikiLinkTraking($Page['body'])) {
			//print_r($ListeDesLiens); exit;
			foreach ($ListeDesLiens as $UnLien) {
				// Concatner les valeurs, sauf si elle page pointe vers elle-mme
				if ($tag != $UnLien)
    				// Portion fixe de la requte d'insetion
    				$SQL .= "INSERT INTO ".$this->GetConfigValue("table_prefix")."links (from_tag, to_tag) VALUES ";
					$SQL .= "('".$this->dbObject->escape_string($tag)."','"
							.$this->dbObject->escape_string($UnLien)."');";
			}
			
			// On vrifie qu'en cas de lien unique  crire,
			// il ne pointait pas vers la page courante (lien jamais crit)
			if (isset($SQL)) {
				// Ecrire la table
				$this->QueryOnly($SQL);
			}
		}
	}
	
	function Header()
	{
		$name = WIKINI_HEADER;

		if (!isset($this->actionHeader[$name])) {
			$filename = $name . PHP_EX;

			if ($actionObj = $this->IncludeBuffered($filename, null, null, WIKINI_INCLUDE_PATH)) $this->actionHeader[$name] = &$actionObj;
			else return '<i>Header invalide &quot;' . htmlspecialchars(WIKINI_INCLUDE_PATH . $filename) . '&quot;</i>';
		}

		return $this->actionHeader[$name];
	}
	
	function Footer()
	{
		$name = WIKINI_FOOTER;

		if (!isset($this->actionFooter[$name])) {
			$filename = $name . PHP_EX;

			if ($actionObj = $this->IncludeBuffered($filename, null, null, WIKINI_INCLUDE_PATH)) $this->actionFooter[$name] = &$actionObj;
			else return '<i>Footer invalide &quot;' . htmlspecialchars(WIKINI_INCLUDE_PATH . $filename) . '&quot;</i>';
		}

		return $this->actionFooter[$name];
	}



	// FORMS
	
	function FormOpen($method = "", $tag = "", $formMethod = "post", $divclass = "")
	{
		$result = "<form action=\"".$this->href($method, $tag)."\" method=\"".$formMethod."\">\n";
		if ($divclass != "") $divclass = ' class="'.trim($divclass).'"';
		$result .= "\t<div$divclass>\n";
		if (!$this->GetConfigValue("rewrite_mode")) $result .= "\t\t<input type=\"hidden\" name=\"wiki\" value=\"".$this->MiniHref($method, $tag)."\" />\n";

		return $result;
	}

	function FormClose()
	{
		return "\t</div>\n</form>\n";
	}



	// INTERWIKI STUFF
	
	function ReadInterWikiConfig()
	{
		if ($lines = file("interwiki.conf"))
		{
			foreach ($lines as $line)
			{
				if ($line = trim($line))
				{
					list($wikiName, $wikiUrl) = explode(" ", trim($line));
					$this->AddInterWiki($wikiName, $wikiUrl);
				}
			}
		}
	}

	function AddInterWiki($name, $url)
	{
		$this->interWiki[strtolower($name)] = $url;
	}

	function GetInterWikiUrl($name, $tag)
	{
		if (isset($this->interWiki[strtolower($name)])) return $this->interWiki[strtolower($name)].$tag;
		else return FALSE;
	}



	// REFERRERS
	
	function LogReferrer($tag = "", $referrer = "")
	{
		// fill values
		if (!$tag = trim($tag)) $tag = $this->GetPageTag();
		if (!$referrer = trim($referrer) AND isset($_SERVER["HTTP_REFERER"])) $referrer = $_SERVER["HTTP_REFERER"];
		
		// check if it's coming from another site
		if ($referrer && !preg_match("/^".preg_quote($this->GetConfigValue("base_url"), "/")."/", $referrer))
		{
			// avoid XSS (with urls like "javascript:alert()" and co)
			// by forcing http/https prefix
			// NB.: this does NOT exempt to htmlspecialchars() the collected URIs !
			if (!preg_match('`^https?://`', $referrer)) return;

			$this->QueryOnly("INSERT INTO ".$this->GetConfigValue("table_prefix")."referrers".
			    "(page_tag,referrer,time) ".
				"VALUES ('".$this->dbObject->escape_string($tag)."', ".
				"'".$this->dbObject->escape_string($referrer)."', ".
				"'".date('Y-m-d H:i:s')."')");
		}
	}

	function LoadReferrers($tag = "")
	{
		return $this->LoadAll("SELECT referrer, COUNT(referrer) AS num "
			."FROM ".$this->GetConfigValue("table_prefix")."referrers ".($tag = trim($tag) ? 
			"WHERE page_tag = '".$this->dbObject->escape_string($tag)."'" : "")." "
			."GROUP BY referrer "
			."ORDER BY num DESC");
	}

	function PurgeReferrers()
	{
		if ($days = $this->GetConfigValue("referrers_purge_time")) {
			$datesub = date('Y-m-d H:i:s', 
                    mktime(date('H'), date('i'), date('s'),
                           date('m'), date('d')-$days, date('Y')));
			$this->QueryOnly("DELETE FROM ".$this->GetConfigValue("table_prefix")."referrers "
				."WHERE time < '$date_sub'");
		}
	}



	// PLUGINS
	
	/**
	 * Exacutes an "action" module and returns the generated output
	 * @param string $action The name of the action and its eventual parameters,
	 * AS it appears in the page between "{{" and "}}"
	 * @param array $vars An array of additionnal parameters to give to the action, in the form
	 * array( 'param' => 'value').
	 * This allows you to call Action() internally, setting $action to the name of the action
	 * you want to call and it's parameters in an array, wich is more efficient than
	 * the pattern-matching algorithm used to extract the parameters from $action.
	 * @return The output generated by the action.
	 */
	function Action($action, $vars = array())
	{
		$cmd = trim($action);
		// extract $action and $vars_temp ("raw" attributes)
		if (!preg_match("/^([a-zA-Z-0-9]+)\/?(.*)$/", $cmd, $matches))
		{
			return '<i>Action invalide &quot;' . htmlspecialchars($cmd) . '&quot;</i>';
		}
		list(, $action, $vars_temp) = $matches;
		$vars[$vars_temp] = $vars_temp; // usefull for {{action/vars_temp}}
		
		// now that we have the action's name, we can check if the user satisfies the ACLs
		if (!$this->CheckModuleACL($action, 'action'))
		{
			return 'Erreur: vous n\'avez pas acc&egrave;s &agrave; l\'action ' . $action . '.';
		}
		
		// match all attributes (key and value)
		// prepare an array for extract() to work with (in $this->IncludeBuffered())
		if (preg_match_all("/([a-zA-Z0-9]*)=\"(.*)\"/U", $vars_temp, $matches))
		{
			for ($a = 0; $a < count($matches[1]); $a++)
			{
				$vars[$matches[1][$a]] = $matches[2][$a];
			}
		}
		
		$themeMgrObj = &$this->GetThemeManager();
		$themeMgrObj->addActionCSS($action);	// request action CSS file
		if ($actionObj = &$this->GetActionObject($action))
		{
			if (is_object($actionObj))
			{
				$result = $actionObj->PerformAction($vars, $cmd);
			}
			else // $actionObj is an error message
			{
				$result = $actionObj;
			}
		}
		else // $actionObj == null (not found, no error message)
		{
			$this->parameter = &$vars;
			$result = $this->IncludeBuffered(strtolower($action) . PHP_EX, "<i>Action inconnue &quot;$action&quot;</i>", $vars, $this->GetConfigValue("action_path"));
			unset($this->parameter);
		}

		return $result;
	}
	
	/**
	 * Finds the object corresponding to an action, if it exists.
	 * @param string $name The name of an action (should be alphanumeric)
	 * @return mixed
	 * 	- null if the corresponding file was not found or the corresponding class didn't exist after inclusion
	 *  - an error string if the corresponding file was found but an error append while loading it
	 *  - the object corresponding to this action if no problem happend
	 * To check the result, you should use is_object() on it.
	 * You should always assign the result of this method by referrence
	 * to avoid copying the object, which is the default beheviour in PHP4.
	 * @example
	 * $var = &$wiki->GetActionObject('actionname');
	 * if (is_object($var))
	 * {
	 * 		// normal behaviour
	 * }
	 * elseif ($var) // $var is not an object but an error string
	 * {
	 * 		// threat error
	 * }
	 * else // action was not found
	 * {
	 * 		// rescue from inexising action or sth
	 * } 
	 */
	function &GetActionObject($name)
	{
		$name = strtolower($name);
		$actionObj = null; // the generated object
		if (!preg_match('/^[a-z0-9]+$/', $name)) // paranoiac
		{
			return $actionObj;
		}
		
		// already tried to load this object ? (may be null)
		if (isset($this->actionObjects[$name]))
		{
			return $this->actionObjects[$name];
		}
		
		// object not loaded, try to load it
		$filename = $name . '.class.inc' . PHP_EX;
		// load parent class for all action objects (only once)
		require_once WIKINI_INCLUDE_PATH . 'action.class.inc' . PHP_EX;
		// include the action file, this should return an empty string
		$result = $this->IncludeBuffered($filename, null, null, $this->GetConfigValue('action_path'));
		if ($result) // the result was not an empty string, certainly an error message
		{
			$actionObj = $result;
		}
		elseif ($result !== false) // the result was empty but the file was found
		{
			$class = 'Action' . ucfirst($name);
			if (class_exists($class))
			{
				$actionObj = &new $class($this);
				if (!is_a($actionObj, 'WikiniAction'))
				{
					die("Action invalide '$name': classe incorrecte");
				}
			}
		}
		$this->actionObjects[$name] = &$actionObj;

		return $actionObj;
	}
	
	/**
	 * Retrieves the list of existing actions
	 * @return array An unordered array of all the available actions.
	 */
	function GetActionsList()
	{
		$action_path = $this->GetConfigValue('action_path');
		$list = array();
		if ($dh = opendir($action_path))
		{
			while (($file = readdir($dh)) !== false)
			{
				if (preg_match('/^([a-zA-Z-0-9]+)(.class.inc)?' . PHP_EX . '$/', $file, $matches))
				{
					$list[] = $matches[1];
				}
			}
		}

		return $list;
	}
	
	function Method($method)
	{
		$methodLocation = $method . PHP_EX;

		return $this->IncludeBuffered($methodLocation, "<i>M&eacute;thode inconnue \"$methodLocation\"</i>", "", $this->GetConfigValue("handler_path"));
	}

	function Format($text, $formatter = "wakka")
	{
		return $this->IncludeBuffered($this->GetConfigValue("formatter_path").$formatter.PHP_EX, "<div class=\"error\">\n\t<p>Impossible de trouver le formateur \"$formatter\"</p>\n</div>\n", compact("text")); 
	}



	// USERS
	
	function LoadUser($name, $password = 0)
	{
		return $this->LoadSingle("SELECT * "
				."FROM ".$this->GetConfigValue("table_prefix")."users "
				."WHERE name = '".$this->dbObject->escape_string($name)."' ".($password === 0 ? "" : 
				"AND password = '".$this->dbObject->escape_string($password)."'")
				." LIMIT 1");
	}
	
	function LoadUsers()
	{
		return $this->LoadAll("SELECT * "
				."FROM ".$this->GetConfigValue("table_prefix")."users "
				."ORDER BY name");
	}
	
	function GetUserName()
	{
		if ($user = $this->GetUser())
			$name = $user["name"];
		else if (!$name = gethostbyaddr($_SERVER["REMOTE_ADDR"]))
			$name = $_SERVER["REMOTE_ADDR"];
		
		return $name;
	}
	
	function GetUser()
	{
		return (isset($_SESSION["user"]) ? $_SESSION["user"] : '');
	}
	
	function SetUser($user, $remember=0) {
		$_SESSION["user"] = $user;
		$this->SetPersistentCookie("name", $user["name"], $remember);
		$this->SetPersistentCookie("password", $user["password"], $remember);
		$this->SetPersistentCookie("remember", $remember, $remember);
	}
	
	function LogoutUser()
	{
		$_SESSION["user"] = "";
		$this->DeleteCookie("name");
		$this->DeleteCookie("password");
	}
	
	function UserWantsComments()
	{
		if (!$user = $this->GetUser()) return false;
		
		return ($user["show_comments"] == "Y");
	}
	
	function GetParameter($parameter, $default = '')
	{
		return (isset($this->parameter[$parameter]) ? $this->parameter[$parameter] : $default);
	}

	
	// COMMENTS
	
	/**
	 * Charge les commentaires relatifs  une page.
	 * 
	 * @param string $tag Nom de la page. Ex : "PagePrincipale"
	 * @return array Tableau contenant tous les commentaires et leurs
	 * proprits correspondantes.
	 */
	function LoadComments($tag)
	{
		return $this->LoadAll("SELECT * " .
					"FROM ".$this->GetConfigValue("table_prefix")."pages " .
					"WHERE comment_on = '".$this->dbObject->escape_string($tag)."' " .
					"AND latest = 'Y' " .
					"ORDER BY ".$this->dbObject->substr_name()."(tag, 8, length(tag)) + 0 DESC");
	}

	/**
	 * Charge les derniers commentaires de toutes les pages.
	 * 
	 * @param int $limit Nombre de commentaires chargs.
	 *                   0 par dfaut (ie tous les commentaires).
	 * @return array Tableau contenant chaque commentaire et ses
	 *               proprits associes.
	 * @todo Ajouter le paramtre $start pour permettre une pagination
	 *       des commentaires : ->LoadRecentComments(10, 10)
	 */
	function LoadRecentComments($limit = 0)
	{
		// The part of the query which limit the number of comments  
		if(is_numeric($limit) && $limit > 0) $lim = " LIMIT ".$limit;
		else $lim = "";
		
		// Query
		return $this->LoadAll("SELECT * " .
					"FROM " . $this->GetConfigValue("table_prefix") . "pages " .
					"WHERE comment_on != '' " .
					"AND latest = 'Y' " .
					"ORDER BY time DESC " .
					$lim);
	}

	function LoadRecentlyCommentedNew($limit = 50)
	{
		$pageTable = $this->config["table_prefix"]."pages";
		$sql = "SELECT p1.comment_on as tag, p1.tag as comment_tag, p1.time as comment_time, p1.user as comment_user ";
		$sql .= "FROM $pageTable p1 ";
		$sql .= "WHERE p1.time = (SELECT max(p2.time) FROM $pageTable p2 ";
		$sql .=                  "WHERE p2.comment_on = p1.comment_on ";
		$sql .=                  "AND p2.time = (SELECT min(p3.time) FROM $pageTable p3 ";
		$sql .=                                  "WHERE p3.tag = p2.tag)) ";
		$sql .= "AND p1.comment_on <> \"\" ";
		$sql .= "ORDER BY p1.time DESC LIMIT 0, $limit";
		$pages = $this->LoadAll($sql);
		return $pages;
	}

	function LoadRecentlyCommented($limit = 50)
	{
		$pages = array();
		
		// NOTE: this is really stupid. Maybe my SQL-Fu is too weak, but apparently there is no easier way to simply SELECT
		//       all comment pages sorted by their first revision's (!) time. ugh!
		
		// load ids of the first revisions of latest comments. err, huh?
		if ($ids = $this->LoadAll("SELECT MIN(id) AS id "
					."FROM ".$this->GetConfigValue("table_prefix")."pages "
					."WHERE comment_on != '' "
					."GROUP BY tag "
					."ORDER BY id DESC"))
		{
			// load complete comments
			$num=0;
			foreach ($ids as $id)
			{
				$comment = $this->LoadSingle("SELECT * "
							."FROM ".$this->GetConfigValue("table_prefix")."pages "
							."WHERE id = '".$id["id"]."' ");

				// Put pages (comments) in cache
				$this->Cache->CachePage($comment);

				if (!isset($comments[$comment["comment_on"]]) && $num < $limit)
				{
					$comments[$comment["comment_on"]] = $comment;
					$num++;
				}
			}
		
			// now load pages
			if ($comments)
			{
				// now using these ids, load the actual pages
				foreach ($comments as $comment)
				{
					$page = $this->LoadPage($comment["comment_on"]);
					$page["comment_user"] = $comment["user"];
					$page["comment_time"] = $comment["time"];
					$page["comment_tag"] = $comment["tag"];
					$pages[] = $page;
				}
			}
		}
		// load tags of pages 
		//return $this->LoadAll("SELECT comment_on AS tag, max(time) AS time, tag AS comment_tag, user from ".$this->GetConfigValue("table_prefix")."pages WHERE comment_on != '' GROUP BY comment_on ORDER BY time DESC");
		return $pages;
	}



	// ACCESS CONTROL
	
	// returns true if logged in user is owner of current page, or page specified in $tag
	function UserIsOwner($tag = "")
	{
		// check if user is logged in
		if (!$this->GetUser()) return FALSE;

		// set default tag
		if (!$tag = trim($tag)) $tag = $this->GetPageTag();

		// If the page doesn't exist, the user isn't the owner
		if ($this->PageExists($tag) == FALSE) return FALSE;
		else {
			// check if user is owner
			if ($this->GetPageOwner($tag) == $this->GetUserName()) return TRUE;
		}
	}

	/**
	 * @param string $group The name of a group
	 * @return string the ACL associated with the group $gname
	 */
	function GetGroupACL($group)
	{
		return $this->GetTripleValue($group, WIKINI_VOC_ACLS, GROUP_PREFIX);
	}
	
	/**
	 * @param string $group The name of a group
	 * @return boolean true iff the user is in the given $group 
	 */
	function UserIsInGroup($group, $user = null)
	{
		return $this->CheckACL($this->GetGroupACL($group), $user);
	}
	
	function UserIsAdmin()
	{
		return $this->UserIsInGroup("admin");
	}
	
	function GetPageOwner($tag = "", $time = "")
	{
		if (!$tag = trim($tag)) $tag = $this->GetPageTag();
		if ($page = $this->LoadPage($tag, $time)) return $page["owner"];
	}
	
	function SetPageOwner($tag, $user)
	{
		// check if user exists
		if (!$this->LoadUser($user)) return;
		
		// updated latest revision with new owner
		$this->QueryOnly("UPDATE ".$this->GetConfigValue("table_prefix")."pages "
					."SET owner = '".$this->dbObject->escape_string($user)."' "
					."WHERE tag = '".$this->dbObject->escape_string($tag)."' "
					."AND latest = 'Y' ");

		$this->Cache->UnCachePage($tag);
	}
	
	function LoadAcl($tag, $privilege, $useDefaults = 1)
	{
		if ((!$acl = $this->LoadSingle("SELECT * "
					."FROM ".$this->GetConfigValue("table_prefix")."acls "
					."WHERE page_tag = '".$this->dbObject->escape_string($tag)."' "
					."AND privilege = '".$this->dbObject->escape_string($privilege)
                    ."' "))	&& $useDefaults)
		{
			$acl = array("page_tag" => $tag,
				"privilege" => $privilege,
				"list" => $this->GetConfigValue("default_".$privilege."_acl"));
		}

		return $acl;
	}
	
	function SaveAcl($tag, $privilege, $list)
	{
		if ($this->LoadAcl($tag, $privilege, 0)) $this->QueryOnly("UPDATE ".$this->GetConfigValue("table_prefix")."acls "
			."SET list = '".$this->dbObject->escape_string(trim(str_replace("\r", "", $list)))."' "
			."WHERE page_tag = '".$this->dbObject->escape_string($tag)."'"
			." AND privilege = '".$this->dbObject->escape_string($privilege)."' ");
		else $this->QueryOnly("INSERT INTO ".$this->GetConfigValue("table_prefix")."acls"
		    ."(list, page_tag, privilege) "  
			."VALUES ('".$this->dbObject->escape_string(trim(str_replace("\r", "", $list)))."',"
			."'".$this->dbObject->escape_string($tag)."',"
			."'".$this->dbObject->escape_string($privilege)."')");
	}

	// returns true if $user (defaults to current user) has access to $privilege on $page_tag (defaults to current page)
	function HasAccess($privilege, $tag = "", $user = "")
	{
  		if (($privilege == 'comment') && !$this->GetConfigValue('comment_enable')) return FALSE;

		if (!$user = trim($user))
		{
			// if current user is owner, return true. owner can do anything!
			if ($this->UserIsOwner($tag)) return TRUE;
			else $user = $this->GetUserName();
		}

		// set defaults
		if (!$tag = trim($tag)) $tag = $this->GetPageTag();

		// If the page don't exist, don't load ACL from base
		if ($this->PageExists($tag) == FALSE) {
			// Add a comment to an inexisting page is crazy !
			if ($privilege == "comment") {
				$acl = array("page_tag" => $tag,
					"privilege" => $privilege,
					"list" => "");

			}
			// Default ACL
			else {
				$acl = array("page_tag" => $tag,
					"privilege" => $privilege,
					"list" => $this->GetConfigValue("default_".$privilege."_acl"));
			}
		}
		else {
			// load acl
			$acl = $this->LoadAcl($tag, $privilege);
		}

		// fine fine... now go through acl
		return $this->CheckACL($acl["list"], $user);
	}
	
	/**
	 * Checks if some $user satisfies the given $acl
	 * @param string $acl The acl to check, in the same format than for pages ACL's
	 * @param string $user The name of the user that must satisfy the ACL. By default
	 * the current remote user.
	 * @return bool True if the $user satisfies the $acl, false otherwise
	 */
	function CheckACL($acl, $user = null)
	{
		if (!$user)
		{
			$user = $this->GetUserName();
		}
		
		foreach (explode("\n", $acl) as $line)
		{
			$line = trim($line);

			// check for inversion character "!"
			if (preg_match("/^[!](.*)$/", $line, $matches))
			{
				$negate = 1;
				$line = $matches[1];
			}
			else
			{
				$negate = 0;
			}

			// if there's still anything left... lines with just a "!" don't count!
			if ($line)
			{
				switch ($line[0])
				{
				// comments
				case "#":
					break;
				// everyone
				case "*":
					return !$negate;
				// registered users
				case "+":
					if (!$this->LoadUser($user)) 
					{
						return $negate;
					}
					else
					{
						return !$negate;
					}
				// groups
				case '@':
					$gname = substr($line, 1);
					// paranoiac: avoid line = '@'
					if ($gname && $this->UserIsInGroup($gname, $user))
					{
						return !$negate;
					}
					break;
				// simple user entry				
				default:
					if ($line == $user)
					{
						return !$negate;
					}
				}
			}
		}
		
		// tough luck.
		return false;
	}
	
	/**
	 * Loads the module ACL for a certain module
	 * @param string $module The name of the module
	 * @param string $module_type The type of module: 'action' or 'handler'
	 * @return mixed The ACL for the given module or <tt>null</tt> if no such
	 * ACL was found (which should probably be interpreted AS '*'). 
	 */
	function GetModuleACL($module, $module_type)
	{
		$module = strtolower($module);
		switch ($module_type)
		{
			case 'action':
				return $this->GetTripleValue($module, WIKINI_VOC_ACLS, WIKINI_VOC_ACTIONS_PREFIX);
				// TODO allow actions to have default ACL's ? (defined in action objects)
			case 'handler':
				return $this->GetTripleValue($module, WIKINI_VOC_ACLS, WIKINI_VOC_HANDLERS_PREFIX);
			default:
				return null; // TODO error msg ?
		}
	}
	
	/**
	 * Sets the $acl for a given $module
	 * @param string $module The name of the module
	 * @param string $module_type The type of module ('action' or 'handler')
	 * @param string $acl The new ACL for that module
	 * @return 0 on success, > 0 on error (see InsertTriple and UpdateTriple)
	 */
	function SetModuleACL($module, $module_type, $acl)
	{
		$module = strtolower($module);
		$voc_prefix = $module_type == 'action' ? WIKINI_VOC_ACTIONS_PREFIX : WIKINI_VOC_HANDLERS_PREFIX;
		$old = $this->GetTripleValue($module, WIKINI_VOC_ACLS, $voc_prefix);
		if ($old === null)
		{
			return $this->InsertTriple($module, WIKINI_VOC_ACLS, $acl, $voc_prefix);
		}
		elseif ($old === $acl)
		{
			return 0; // nothing has changed
		}
		else
		{
			return $this->UpdateTriple($module, WIKINI_VOC_ACLS, $old, $acl, $voc_prefix);
		}
	}

	/**
	 * Checks if a $user satisfies the ACL to access a certain $module
	 * @param string $module The name of the module to access
	 * @param string $module_type The type of the module ('action' or 'handler')
	 * @param string $user The name of the user. By default
	 * the current remote user.
	 * @return bool True if the $user has access to the given $module, false otherwise.
	 */
	function CheckModuleACL($module, $module_type, $user = null)
	{
		$acl = $this->GetModuleACL($module, $module_type);
		if ($acl === null) return true; // undefined ACL means everybody has access
		return $this->CheckACL($acl, $user);
	}


	// MAINTENANCE
	
	function Maintenance() {

		// purge referrers
		$this->PurgeReferrers();

		// purge old page revisions
		$this->PurgePages();
	}



	// THE BIG EVIL NASTY ONE!
	
	function Run($tag, $method = "")
	{
		// Si aucune mthode n'est spcifie, on se rabat sur l'affichage
		if (!$this->method = trim($method)) $this->method = "show";

		$tag = trim($tag);

		// Si aucun tag n'est mentionn, rediriger vers la page d'accueil
		if ($tag == '') $this->Redirect($this->href('', $this->GetConfigValue("root_page")));

		if ($this->PageExists($tag) == TRUE) {
			// On vrifie si la casse est correcte
			if ($this->PageExists($tag, TRUE) == FALSE)
			{
				// Redirection vers la page de casse correcte avec la mme mthode
				$this->Redirect($this->href($this->method != 'show' ? $this->method : '', $this->GetExistingPageTag($tag)));
			}

			// Chargement de la page
			$this->SetPage($this->LoadPage($tag, (isset($_REQUEST["time"]) ? $_REQUEST["time"] :'')));
		}
		else {
			// Si la syntaxe du tag est juste
			if ($this->IsWikiName($tag) == TRUE) {

				// Initialisation d'une page vide
				$page = array();
				$page["id"] = 0;
				$page["tag"] = $tag;
				$page["body"] = '';
				$page["time"] = '';
				$page["latest"] = 'Y';
				$page["comment_on"] = '';

				$this->SetPage($page);
			}
			else $this->Redirect($this->href('', $this->GetConfigValue("root_page")));

		}
		
		if ((!$this->GetUser()
			&& isset($_COOKIE["name"]))
				&& ($user = $this->LoadUser($_COOKIE["name"], $_COOKIE["password"]))) 
		{
			$this->SetUser($user, $_COOKIE["remember"]);
		}
		
		if(!mt_rand(0,99)) $this->Maintenance();
		
		$this->LogReferrer();
		$this->ReadInterWikiConfig();

		//correction pour un support plus facile de nouveaux handlers
		$themeMgrObj = &$this->GetThemeManager();
		$themeMgrObj->resetCSS($this->method);	// default BODY tag class = handler name
		echo $this->Method($this->method);
			
		// action redirect: aucune redirection n'a eu lieu, effacer la liste des redirections prcdentes
		if(!empty($_SESSION['redirects'])) session_unregister('redirects');
	}
}

?>
